<?php
namespace PME\Controllers\Relatorios;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;

class EmissaoCardDiaController extends \Zeedhi\Framework\Controller\Report {

    protected $entityManager;
    protected $birtStrategy;
    protected $utilities;

    public function __construct(ReportService $reportService, $entityManager, $birtStrategy, $utilities) {
        $this->entityManager = $entityManager;
        $this->birtStrategy = $birtStrategy;
        $this->utilities = $utilities;

        parent::__construct($reportService);
    }

    protected $reportMapping = array (
      'PME03100' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
            "FILNDPCAL"    => "P_FILNDPCAL",
            "IMPNUTR"      => "P_NUTRI",
            "IMPNDPCAL"    => "P_NDPCAL",
            "CDFILIAL"     => "P_CDFILIAL",
            "NMFILIAGRE"   => "P_NMFILIAGRE",
            'SGUNIDADE1'   => 'P_SGUNIDADE1',
            'SGUNIDADE2'   => 'P_SGUNIDADE2',
            'SGUNIDADE3'   => 'P_SGUNIDADE3',
            'SGUNIDADE4'   => 'P_SGUNIDADE4',
            'NMREDUZIDO1'  => 'P_NMREDUZIDO1',
            'NMREDUZIDO2'  => 'P_NMREDUZIDO2',
            'NMREDUZIDO3'  => 'P_NMREDUZIDO3',
            'NMREDUZIDO4'  => 'P_NMREDUZIDO4',
            'QUEBRA'       => 'P_QUEBRA',
            'CDFILCENTPME' => 'P_CDFILCENTPME',
            'NRCARDPME'    => 'P_NRCARDPME',
            'DTDE'         => 'P_DTINICIAL',
            'DTATE'        => 'P_DTFINAL',
            'CDSERVFILI'   => 'P_CDSERVFILI',
            'TODOS'        => 'P_TODOS',
            'NRSEQUNUTR'   => 'P_NRSEQUNUTR',
            'ORDENA'       => 'P_ORDENA',
            'IMPMSG'       => 'P_IMPMSG',
            'NRORG'        => 'P_NRORG',
            'CONSPRAT'     => 'P_PRATOOPCAONUTR',
            'NMUNIDADE'    => 'P_NMUNIDADE',
            'IMPCOMENSAIS' => 'P_IMPCOMENS'
            )
        ),
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $nrorg = $this->utilities->getNrOrg();
        
        if(empty($row['UNIDADE'])){
          $cdfilial = $row['CDFILCENTPME'];  
          $row['NMUNIDADE'] = "";
        } else{ 
            $cdfilial = is_array($row['CDFILIAL']) ? join('_', $row['CDFILIAL']) : $row['CDFILIAL'];
            $row['NMUNIDADE'] = $row['UNIDADE']['CDFILIAL'].' '.$row['UNIDADE']['NMFILIAL'];
        }
        
        $row['CDFILIAL'] = $cdfilial;
        $row['FILNDPCAL'] = $cdfilial;

        $nutrientes = $this->getNutrientes($cdfilial);

        $nrSeqNutr = '';
        if($row['IMPNUTR'] == 'S') $nrSeqNutr = $this->procedureCalcNutr($row, $nutrientes);
        
        $row['NMREDUZIDO1'] = $nutrientes[0]["NMREDUZIDO1"];
        $row['NMREDUZIDO2'] = $nutrientes[0]["NMREDUZIDO2"]; 
        $row['NMREDUZIDO3'] = $nutrientes[0]["NMREDUZIDO3"];
        $row['NMREDUZIDO4'] = $nutrientes[0]["NMREDUZIDO4"];

        $row['SGUNIDADE1']  = $nutrientes[0]["SG1"];
        $row['SGUNIDADE2']  = $nutrientes[0]["SG2"];
        $row['SGUNIDADE3']  = $nutrientes[0]["SG3"];
        $row['SGUNIDADE4']  = $nutrientes[0]["SG4"];
        
        $row['NRSEQUNUTR'] = $nrSeqNutr;

        if(count($row['CDSERVFILI']) == 0) {
            $row['TODOS'] = 'T';
            $row['CDSERVFILI'] = '';
        } else {
          $row['TODOS'] = null;
          $row['CDSERVFILI'] = is_array($row['CDSERVFILI']) ? join("_", $row['CDSERVFILI']) : $row['CDSERVFILI'];
        }
      
        $row['NRORG'] = $nrorg;
        $row['CONSPRAT'] = empty($row['CONSPRAT']) ? 'N' : 'S';
        $row['QUEBRA'] = empty($row['QUEBRA']) ? 'N' : 'S';
        
        $this->birtStrategy->setReportLocale($this->utilities->getLanguage());
        $this->birtStrategy->setReportFormat(strtolower($row['reportFormat']));
    }

    private function procedureCalcNutr($row, $nutrientes){

      if(count($row['SERVICO']) == 0) $cdservfili = 'T';
      else $cdservfili = join('_', $row['SERVICO']);

      $listaNutri = '';
      
      if($nutrientes[0]["CDNUTRIENTE1"])  $listaNutri .= $nutrientes[0]["CDNUTRIENTE1"]."_";
      if($nutrientes[0]["CDNUTRIENTE2"])  $listaNutri .= $nutrientes[0]["CDNUTRIENTE2"]."_";
      if($nutrientes[0]["CDNUTRIENTE3"])  $listaNutri .= $nutrientes[0]["CDNUTRIENTE3"]."_";
      if($nutrientes[0]["CDNUTRIENTE4"])  $listaNutri .= $nutrientes[0]["CDNUTRIENTE4"]."_";

      $nrSeqNutr = $this->utilities->callProcedureNutrCard($row['CDFILCENTPME'], $row['CDFILIAL'], $cdservfili, $listaNutri, $row['IMPNDPCAL'], $row['DTDE'], $row['DTATE'], $row['TIPOCARD']);

      return $nrSeqNutr;
    }

    private function getNutrientes ($cdfilial) {
        $sql = "SELECT
                  NUT1.NMREDUZIDO AS NMREDUZIDO1,
                  NUT1.SGUNIDADE  AS SG1,
                  NUT1.CDNUTRIENTE AS CDNUTRIENTE1,
                  NUT2.NMREDUZIDO AS NMREDUZIDO2,
                  NUT2.SGUNIDADE  AS SG2,
                  NUT2.CDNUTRIENTE AS CDNUTRIENTE2,
                  NUT3.NMREDUZIDO AS NMREDUZIDO3,
                  NUT3.SGUNIDADE  AS SG3,
                  NUT3.CDNUTRIENTE AS CDNUTRIENTE3,
                  NUT4.NMREDUZIDO AS NMREDUZIDO4,
                  NUT4.SGUNIDADE  AS SG4,
                  NUT4.CDNUTRIENTE AS CDNUTRIENTE4
              FROM
                  PARAMPLANEJA
              LEFT JOIN NUTRIENTE NUT1 ON NUT1.CDNUTRIENTE = PARAMPLANEJA.CDNUTRRELAT1
              LEFT JOIN NUTRIENTE NUT2 ON NUT2.CDNUTRIENTE = PARAMPLANEJA.CDNUTRRELAT2
              LEFT JOIN NUTRIENTE NUT3 ON NUT3.CDNUTRIENTE = PARAMPLANEJA.CDNUTRRELAT3
              LEFT JOIN NUTRIENTE NUT4 ON NUT4.CDNUTRIENTE = PARAMPLANEJA.CDNUTRRELAT4
              WHERE PARAMPLANEJA.CDFILIAL = :P_CDFILCENTPME
              AND PARAMPLANEJA.NRORG = :P_NRORG";
        $params = array(
            'P_CDFILCENTPME' => $cdfilial,
            'P_NRORG' => $this->utilities->getNrOrg()
        );

        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
}